@@ -1,12 +1,13 @@
 """Tool for automatically drawing images."""
 import logging
 import time
-from typing import Iterator, List, Tuple
+from typing import Iterator, List, Optional, Tuple
 
 import PIL.Image
 
 from pydispix.canvas import Canvas, Pixel
 from pydispix.client import Client
+from pydispix.errors import OutOfBoundaries
 
 logger = logging.getLogger('pydispix')
 
@@ -36,6 +37,29 @@ class AutoDrawer:
         if image_width > canvas_width or image_height > canvas_height:
             raise OutOfBoundaries(f"Can't draw picture bigger than the canvas ({image_size} > {canvas_size})")
 
+    @staticmethod
+    def _grid_from_img(
+        image: PIL.Image.Image,
+        scale: float = 1
+    ):
+        if image.mode == 'RGBA':
+            new_image = PIL.Image.new('RGB', image.size)
+            new_image.paste(image, mask=image)
+            image = new_image
+
+        width = round(image.width * scale)
+        height = round(image.height * scale)
+
+        if scale != 1:
+            image = image.resize((width, height), PIL.Image.BILINEAR)
+
+        data = list(image.getdata())
+        grid = [
+            [Pixel(r, g, b) for r, g, b in data[start:start + width]]
+            for start in range(0, len(data), width)
+        ]
+        return grid
+
     @classmethod
     def load_image(
         cls,
@@ -45,43 +69,9 @@ class AutoDrawer:
         scale: float = 1
     ) -> 'AutoDrawer':
         """Draw from the pixels of an image."""
-        if image.mode == 'RGBA':
-            new_image = PIL.Image.new('RGB', image.size)
-            new_image.paste(image, mask=image)
-            image = new_image
-
-        width = round(image.width * scale)
-        height = round(image.height * scale)
-        resized = image.resize((width, height), PIL.Image.BILINEAR)
-        data = list(resized.getdata())
-        grid = [
-            [Pixel(*pixel) for pixel in data[start:start + width]]
-            for start in range(0, len(data), width)
-        ]
+        grid = cls._grid_from_img(image, scale)
         return cls(client, *xy, grid)
 
-    @classmethod
-    def load(cls, client: Client, data: str) -> 'AutoDrawer':
-        """Draw from a string that specifies the pixels.
-        `data` should be a multi-line string. The first two lines are the x
-        and y coordinates of the top left of the image to draw. The second two
-        are the width and height of the image. The rest of the lines are the
-        pixels of the image, as hex codes (horizontal scanlines, left-to-right
-        top-to-bottom).
-        """
-        lines = data.split('\n')
-        x = int(lines.pop(0))
-        y = int(lines.pop(0))
-        width = int(lines.pop(0))
-        height = int(lines.pop(0))
-        grid = []
-        for _ in range(height):
-            row = []
-            for _ in range(width):
-                row.append(Pixel.from_hex(lines.pop(0)))
-            grid.append(row)
-        return cls(client, x, y, grid)
-
     def _iter_coords(self) -> Iterator[Tuple[int, int]]:
         """Iterate over the coordinates of the image."""
         for x in range(self.x0, self.x1):
@@ -121,5 +111,95 @@ class AutoDrawer:
             canvas = self.client.get_canvas()
 
 
-class OutOfBoundaries(Exception):
-    """Pixel we tried to draw is out of canvas boundaries"""
+class MultiAutoDrawer:
+    """Tool for automatically drawing set of images"""
+    def __init__(
+        self,
+        client: Client,
+        positions: List[Tuple[int, int]],
+        grids: List[List[List[Pixel]]],
+        one_by_one: bool = True
+    ):
+        self.client = client
+        self.drawers = [
+            AutoDrawer(client, *position, grid)
+            for position, grid in zip(positions, grids)
+        ]
+        self.positions_generator = self._one_by_one_positions if one_by_one else self._per_pixel_positions
+
+    @classmethod
+    def load_images(
+        cls,
+        client: Client,
+        positions: List[Tuple[int, int]],
+        images: List[PIL.Image.Image],
+        scales: Optional[List[int]] = None,
+        one_by_one: bool = True
+    ) -> "MultiAutoDrawer":
+        """Draw from pixels on the images."""
+        if scales is None:
+            # Default all scales to 1
+            scales = [1 for _ in images]
+
+        grids = [
+            AutoDrawer._grid_from_img(image, scale)
+            for image, scale in zip(images, scales)
+        ]
+
+        return cls(client, positions, grids, one_by_one)
+
+    def _one_by_one_positions(self) -> Iterator[Tuple[AutoDrawer, Tuple[int, int]]]:
+        """
+        Return iterator of the pixels to check for the passed set of grids (images).
+        This will one by one through the individual images. This allows for prioritizing
+        certain images over others.
+        """
+        coord_generators = [drawer._iter_coords() for drawer in self.drawers]
+        for drawer, coord_generator in zip(self.drawers, coord_generators):
+            for x, y in coord_generator:
+                yield drawer, (x, y)
+
+    def _per_pixel_positions(self) -> Iterator[Tuple[AutoDrawer, Tuple[int, int]]]:
+        """
+        This is similar to `_one_by_one_positions`, except instead of iterating one by one,
+        we are instead iterating through individual pixels, i.e. all 1st pixels from
+        all images, all 2nd pixels, etc.
+        """
+        coord_generators = [drawer._iter_coords() for drawer in self.drawers]
+
+        # Keep track of which drawers are already depleted
+        depleted = {drawer: False for drawer in self.drawers}
+        # Keep running as long as we have any non-depleted drawers
+        while any(is_depleted is False for is_depleted in depleted.values()):
+            for drawer, coord_generator in zip(self.drawers, coord_generators):
+                try:
+                    x, y = next(coord_generator)
+                except StopIteration:
+                    # If we already depleted pixels from this drawer, just ignore
+                    # it and move on to another one
+                    depleted[drawer] = True
+                    continue
+                yield drawer, (x, y)
+
+    def draw(
+        self,
+        guard: bool = False,
+        guard_delay: int = 5,
+        show_progress: bool = True,
+    ):
+        canvas = self.client.get_canvas()
+
+        while True:
+            for drawer, (x, y) in self.positions_generator():
+                if drawer.draw_pixel(canvas, x, y, show_progress=show_progress):
+                    canvas = self.client.get_canvas()
+            if not guard:
+                # Check this here, to act as do-while,
+                # (always run first time, only continue if this is met)
+                break
+            # When we're guarding we need to update canvas even if no pixel was drawn
+            # because otherwise we'd be looping over same non-updated canvas forever
+            # since this looping with no changes takes a long time, we should also sleep
+            # to avoid needless cpu usage
+            time.sleep(guard_delay)
+            canvas = self.client.get_canvas()
